-- led_ctrl.vhd: LED controler - unit architecture
-- Copyright (C) 2009 CESNET
-- Author(s): Libor Polcak <polcak_l@liberouter.org>
--
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions
-- are met:
-- 1. Redistributions of source code must retain the above copyright
--    notice, this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright
--    notice, this list of conditions and the following disclaimer in
--    the documentation and/or other materials provided with the
--    distribution.
-- 3. Neither the name of the Company nor the names of its contributors
--    may be used to endorse or promote products derived from this
--    software without specific prior written permission.
--
-- This software is provided ``as is'', and any express or implied
-- warranties, including, but not limited to, the implied warranties of
-- merchantability and fitness for a particular purpose are disclaimed.
-- In no event shall the company or contributors be liable for any
-- direct, indirect, incidental, special, exemplary, or consequential
-- damages (including, but not limited to, procurement of substitute
-- goods or services; loss of use, data, or profits; or business
-- interruption) however caused and on any theory of liability, whether
-- in contract, strict liability, or tort (including negligence or
-- otherwise) arising in any way out of the use of this software, even
-- if advised of the possibility of such damage.
--
-- $Id$
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

-- ----------------------------------------------------------------------------
--                      Architecture declaration
-- ----------------------------------------------------------------------------
architecture led_ctrl_arch of led_ctrl is

   signal cntr       : std_logic_vector(CNTR_SIZE-1 downto 0);

begin

   cntrp: process(CLK)
   begin
      if (CLK'event and CLK='1') then
         if (LINE_UP = '0') or (RESET = '1') then
            cntr <= (others => '1');
         elsif (PACKET = '1') or (cntr /= 0) then
            cntr <= cntr + 1;
         end if;
      end if;
   end process;

   LED      <= cntr(cntr'high - 2);

   BLINKING <= '0' when (cntr = 0) else
               '1';

end architecture led_ctrl_arch;
